function [SKM]=learn_SKM(output_dir,filename_prefix,C0,Y,train,test,varargin)
%learn_SKM(output_dir,filename_prefix,C0,Y,train,test,varargin)
%input arg 
%   output_dir: name of the directory where the output is to be
%   stored. If empty, the results are not written but just returned
%   in the SKM structure.  
%   filename_prefix: is the prefix of the files output by the code
%   in <output dir> 
%   C0: The regularisation value(s): 
%          - if C0 is scalar, the same regularization is used for all
%          data points
%          - if C0 is a pair of values, the first regularization is
%          applied to the positives (against FN) and the second is
%          applied to the negatives (against FP)
%          - C0 can also be a full vector of the same size as the
%          training set or the full set.
%   Y: is assumed to be a column vector of the same length as the
%   number of entries in the kernel matrices, containing the
%   {+1,-1} labels (only training are required, test are used for
%   error measure on test)
%   train: is the list of the indices of the training points in Y
%   and the kernel matrices
%   test: is the list of the indices of the testing points in Y and
%   in the kernels matrices
%   k_1,...,k_m the names of the files containing the kernels with
%   the following format assumed: each file is tab separated with
%   first row and colums containing headers that are ignored by the code. 
%
%output arg
%       the only ouput to the matlab command line is the SKM
%       object (see documentation). If <output_dir>
global K_stack DKDa aDKDa

N=length(Y);

if N<3,
  error(['The number of datapoints is expected to be greater than 3 but ' ...
      ' Y has only ' num2str(N) ' entries']);
end

mypath=output_dir;
kernel_list=varargin;

nbker=length(kernel_list);
y_train=Y(train);
y_test=Y(test);

ntrain=length(train);
ntest=length(test);

if any(abs(y_train)~=1),
  error('Some labels in the training set are not in {-1,1}')
elseif min(y_train)>-1,
  error('There are no -1 labels in the training set');
elseif max(y_train)<1,
  error('There are no +1 labels in the training set');
end


%% Loading kernels in memory
K_stack=zeros(N*nbker,N);
for i=1:nbker,
  K=dlmread(kernel_list{i},'\t',1,1);
  if size(K,1)~=N;
    error(['Kernel number ' num2str(i) ' has ' num2str(size(K,1)) ' rows ' ...
	   'but Y has ' num2str(N) ' entries']);
  elseif size(K,2)~=N;
    error(['Kernel number ' num2str(i) ' has ' num2str(size(K,2)) ' columns ' ...
	   'but Y has ' num2str(N) ' entries']);
  end 
  K_stack(((i-1)*N+1):(i*N),:)=K;
end


init_parameters;
counters.global=0;


%% Making the C vector
if length(C0)==1,
  C=C0*ones(ntrain,1);
elseif length(C0)==2,
  C=C0(1)*(y_train==1)+C0(2)*(y_train==-1);
elseif length(C0)==ntrain,
  C=C0;
elseif length(C0)==N,
  C=C0(train);
else
  error(['The length of C0 is different from 1, 2, the number of training' ...
       ' points or the total number datapoints']);
end
d=ones(nbker,1);
SKM0=[];
[SKM counters]=reg_SKM_smo2(SKM0,train',y_train,C,d,regu,acc,dkb,counters,option);
%[testerr roc_err distest]=evaluate_skm5(y_test,test,train,SKM,evaluation.use_AUTC);

%% Evaluate the SKM on the testing set and if the labels for the
%testing set are correct calculate the error on the test set.
 
nnzet=length(SKM.etas_tilde);
K_test_train=zeros(ntest,ntrain);
for i=1:nnzet,
    if SKM.etas_tilde(i)~=0,
        K_test_train=K_test_train+SKM.etas_tilde(i)*K_stack(N*(i-1)+test,train);
    end
end
predict=((K_test_train*(y_train.*SKM.alphas))+SKM.bias);
err_test=(1-mean(sign(y_test.*((K_test_train*(y_train.*SKM.alphas))+SKM.bias))))/2;
SKM.discrim_test=predict;
SKM.err_test=err_test;

%% Unless 'output_dir' is empty 
if ~isempty(mypath),
  %% File containing true labels + class probabilities for the train data
  fid=fopen([mypath '/' filename_prefix '.train'],'w');
  for i=1:ntrain,
    fprintf(fid,'%d %d %.4g %.4g\n',train(i),y_train(i),SKM.discrim(i),SKM.alphas(i));
  end
  fclose(fid);

  %% File containing true labels + the raw discriminant of the SVM for the test data
  fid=fopen([mypath '/' filename_prefix '.test'],'w');
  for i=1:ntest,
    fprintf(fid,'%d %d %.4g\n',test(i),y_test(i),SKM.discrim_test(i));
  end
  fclose(fid);
  
  %% File containing the bias
  fid=fopen([mypath '/' filename_prefix '.bias'],'w');
  fprintf(fid,'%g\n',SKM.bias);
  fclose(fid);

  %% File containing the final duality gap for assesment of convergence
  fid=fopen([mypath '/' filename_prefix '.conv'],'w');
  fprintf(fid,'%g\n',SKM.duality_gap);
  fclose(fid);

  %% File containing the weights
  fid=fopen([mypath '/' filename_prefix '.weights'],'w');
  fprintf(fid,'%g\n',SKM.etas_tilde');
  fclose(fid);

  save([mypath '/results'],'SKM','C0');
end
